import numpy as np
from scipy import io, interpolate, stats


def find_photo_z(type, file=None, file_z=None, z=None, mu=None,
                 sigma=None):
    """Type: file or gauss
       Note: file_z implementation NOT tested.
    """

    if type == 'file':
        pz_file = io.readsav(file)
        pz = pz_file['p_z']
        if file_z is not None:
            my_z = io.readsav(file_z)
            my_z = my_z['z']
        else:
            my_z = np.arange(0, 5, .01)
        if np.shape(pz) != np.shape(my_z):
            raise ValueError("pz array and z array are different sizes!")
        func_my_photo_z = interpolate.interp1d(my_z, pz)
        my_photo_z = func_my_photo_z(z)
        my_photo_z = np.asarray(my_photo_z/my_photo_z.max())

    elif type == 'gauss':
        my_photo_z = stats.norm.pdf(z, mu, sigma)
        my_photo_z = np.asarray(my_photo_z/my_photo_z.max())
    return my_photo_z



import os.path
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
from funcsForSimulatedData import load, file_name, mask
from redshiftIterator import save_arrays, iterator


def flux_fluxDiff_arrays(my_dir, file_dir, filter1, filter2, z):

    mc_file_name = file_name(my_dir, file_dir, z)
    mc_dict = load(mc_file_name)
    type_Ia_flux_diff = (mc_dict['type_Ia']['flux'][filter2] -
                         mc_dict['type_Ia']['flux'][filter1])
    type_Ibc_flux_diff = (mc_dict['type_Ibc']['flux'][filter2] -
                          mc_dict['type_Ibc']['flux'][filter1])
    type_II_flux_diff = (mc_dict['type_II']['flux'][filter2] -
                         mc_dict['type_II']['flux'][filter1])
    flux = [mc_dict['type_Ia']['flux'][filter1],
            mc_dict['type_Ibc']['flux'][filter1],
            mc_dict['type_II']['flux'][filter1]]
    diff = [type_Ia_flux_diff, type_Ibc_flux_diff, type_II_flux_diff]
    for i, item in enumerate(diff):
        flux[i] = mask(flux[i], 98)
        diff[i] = mask(diff[i], 98)

    return flux, diff


def contour(my_dir, file_dir, filter1, filter2, flux_filter1,
            flux_filter1_err, flux_filter2, flux_filter2_err, z, outdir):
    """Generates a plot and transforms the figure into an HTML file from the
    montecarlo data. Adds a point to the data"""
    if not os.path.exists(outdir):
        os.makedirs(outdir)

    flux, diff = flux_fluxDiff_arrays(my_dir, file_dir, filter1, filter2, z)
    diff_data = flux_filter2 - flux_filter1
    diff_data_err = flux_filter2_err + flux_filter1_err

    sns.set(style="white", palette="muted")
    sns.set_context("paper", font_scale=1.5, rc={"lines.linewidth": 2.5})
    g = sns.JointGrid(filter2.upper() + " - " + filter1.upper() +
                      " flux difference", filter1.upper() + " flux", space=0)

    sns.kdeplot(diff[0], ax=g.ax_marg_x, legend=True, label='Type Ia')
    sns.kdeplot(flux[0], ax=g.ax_marg_y, vertical=True, legend=False)
    sns.kdeplot(diff[0], flux[0], cut=4, ax=g.ax_joint,
                alpha=0.8, cmap='Blues_d')

    sns.kdeplot(diff[1], ax=g.ax_marg_x, legend=True, label='Type Ib/c')
    sns.kdeplot(flux[1], ax=g.ax_marg_y, vertical=True, legend=False)
    sns.kdeplot(diff[1], flux[1], cut=4, ax=g.ax_joint,
                alpha=0.8, cmap='Greens_d')

    sns.kdeplot(diff[2], ax=g.ax_marg_x, legend=True, label='Type II')
    sns.kdeplot(flux[2], ax=g.ax_marg_y, vertical=True, legend=False)
    sns.kdeplot(diff[2], flux[2], cut=4, ax=g.ax_joint,
                alpha=0.8, cmap='Reds_d')

    g.ax_joint.errorbar(diff_data, flux_filter1,
                        xerr=diff_data_err, yerr=flux_filter1_err,
                        fmt='--o', c='k')
    g.set_axis_labels(xlabel=filter2.upper() + " - " + filter1.upper() +
                      " flux difference (counts/s)",
                      ylabel=filter1.upper() + " flux (counts/s)")
    g.ax_joint.annotate('z=%.2f' % z, xy=(0.75, 0.85),
                        xycoords='axes fraction', size=14)
    plt.tight_layout()
    file_z = 100*z
    file_z = int(file_z)
    # plt.show()
    plt.savefig(outdir + 'z%.0f_' % file_z + filter1 + '_' + filter2 + '_' +
                'contour.png')
    plt.close()


def scatter(my_dir, file_dir, filter1, filter2, z, outdir):

    if not os.path.exists(outdir):
        os.makedirs(outdir)

    flux, diff = flux_fluxDiff_arrays(my_dir, file_dir, filter1, filter2, z)

    sns.set(style="white", palette="muted")
    sns.set_context("paper", font_scale=1.5, rc={"lines.linewidth": 2.5})

    plt.scatter(diff[0], flux[0], c='b',
                label='Type Ia')
    plt.scatter(diff[1], flux[1], c='g',
                label='Type Ib/c')
    plt.scatter(diff[2], flux[2], c='r',
                label='Type II')
    plt.title('SNCosmo Simulated SNe: z=%.2f' % z)
    plt.xlabel(filter2.upper() + " - " + filter1.upper() +
               " flux difference (counts/s)")
    plt.ylabel(filter1.upper() + " flux (counts/s)")
    plt.legend()
    file_z = 100*z
    file_z = int(file_z)
    # plt.show()
    plt.savefig(outdir + 'z%.0f_' % file_z + filter1 + '_' + filter2 + '_' +
                'scatter.png')
    plt.close()


def plot(x, y, ylabel, title, outdir, outname=None):
    sns.set(style="darkgrid", palette="muted")
    sns.set_context("paper", font_scale=1.5, rc={"lines.linewidth": 2.5})
    plot_name = title.replace(' ', '-')
    plot_name = plot_name.lower()
    if outname is not None:
        plot_name = outname

    plt.plot(x, y)
    plt.title(title)
    plt.xlabel('z')
    plt.ylabel(ylabel)
    plt.legend(['Type Ia', 'Type Ib/c', 'Type II'], loc='upper right')
    # plt.show()
    plt.savefig(outdir + plot_name + '.png')
    plt.close()


def subplot(x, y, title, outdir, title_photoz=None):
    ''' x, y and title are arrays
    '''
    new_title = [s for s in title if s]
    # Getting a title for the final PDF composed from the other names
    if sum(1 for x in title if x) == 3:
        final_title = title[0] + ' & ' + title[1] + ' & ' + title[2]
        plot_name = title[0] + '_' + title[1] + '_' + title[2]
    elif sum(1 for x in title if x) == 2:
        final_title = new_title[0] + ' & ' + new_title[1]
        plot_name = new_title[0] + '_' + new_title[1]
    else:
        final_title = new_title[0]
        plot_name = new_title[0]

    # Getting a plot name for the final PDF composed from the other names
    plot_name = plot_name.replace(' ', '-')
    plot_name = plot_name.lower()

    if title_photoz is not None:
        title[2] = title_photoz

    sns.set(style="darkgrid", palette="muted")
    sns.set_context("notebook", font_scale=1.5, rc={"lines.linewidth": 2.5})

    f, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2, sharex='col',
                                               figsize=(12, 12))
    ax1.plot(x[0], y[0])
    ax1.set_title(title[0])
    ax1.legend(['Type Ia', 'Type Ib/c', 'Type II'], loc='upper right')
    ax1.set_ylabel('PDF')

    ax2.plot(x[1], y[1])
    ax2.set_title(title[1])
    ax2.legend(['Type Ia', 'Type Ib/c', 'Type II'], loc='upper right')

    ax3.plot(x[2], y[2])
    ax3.set_title(title[2])
    ax3.legend(['Type Ia', 'Type Ib/c', 'Type II'], loc='upper right')
    ax3.set_xlabel('z')
    ax3.set_ylabel('PDF')

    ax4.plot(x[3], y[3])
    ax4.set_title(final_title)
    ax4.legend(['Type Ia', 'Type Ib/c', 'Type II'], loc='upper right')
    ax4.set_xlabel('z')
    # Saving combined plot
    extent = ax4.get_window_extent().transformed(f.dpi_scale_trans.inverted())
    f.savefig(outdir + plot_name + '_single.png',
              bbox_inches=extent.expanded(1.3, 1.3))

    # plt.show()
    plt.savefig(outdir + plot_name + '.png')
    plt.close()


def combined(final_pdf, my_dir, file_dir, filter1, filter2,
             filter3, flux_filter1, flux_filter2, flux_filter3,
             flux_filter1_err, flux_filter2_err, flux_filter3_err, outdir,
             file_with_RF_and_SF_arrays, photo_z_type=None, photo_z_file=None,
             photo_z_redshift_file=None, mu=None, sigma=None):

    if not os.path.exists(outdir):
        os.makedirs(outdir)

    if not os.path.isfile(file_with_RF_and_SF_arrays):
        file_name = save_arrays(
            my_dir, file_dir, filter1, filter2, filter3, flux_filter1,
            flux_filter2, flux_filter3, flux_filter1_err, flux_filter2_err,
            flux_filter3_err, outdir)
        my_dict = load(file_name)

    else:
        my_dict = load(file_with_RF_and_SF_arrays)
    rf = my_dict['rf']
    sf = my_dict['sf']
    z = my_dict['rf_z']

    rf = np.asarray(rf)
    sf = np.asarray(sf)

    if photo_z_type is not False:
        photo_z, my_z = iterator('photo_z', my_dir, file_dir, filter1,
                                 filter2, filter3, flux_filter1, flux_filter2,
                                 flux_filter3, flux_filter1_err,
                                 flux_filter2_err, flux_filter3_err,
                                 photo_z_type=photo_z_type,
                                 photo_z_file=photo_z_file,
                                 photo_z_redshift_file=photo_z_redshift_file,
                                 mu=mu, sigma=sigma)
        if photo_z_type == 'file':
            title_photoz = 'Galaxy Photo-z from file'
        else:
            title_photoz = ("Galaxy Photo-z: Gaussian( mu=%.2f, sigma=%.2f)"
                            % (mu, sigma))

    if final_pdf == 'RF+SF+photoz':
        plt.clf()
        title = ['Random Forest', 'Survival Function', 'Photo-z']
        plot(z, rf, 'PDF', 'Random Forest', outdir)
        plot(z, sf, '1 - CDF', 'Survival Function', outdir)
        plot(z, photo_z, 'PDF', title_photoz, outdir, outname='photoz')

        first_product = np.multiply(sf, rf)
        product = (np.multiply(first_product.T, photo_z)).T

        x = [z, z, z, z]
        y = [rf, sf, photo_z, product]
        subplot(x, y, title, outdir, title_photoz)
        # norm_product = product/product.sum()

    elif final_pdf == 'RF+SF':
        plt.clf()
        title = ['Random Forest', 'Survival Function', '']
        plot(z, rf, 'PDF', 'Random Forest', outdir)
        plot(z, sf, '1 - CDF', 'Survival Function', outdir)
        product = np.multiply(rf, sf)

        x = [z, z, [], z]
        y = [rf, sf, [], product]
        subplot(x, y, title, outdir)

        # norm_product = product/product.sum()

    elif final_pdf == 'RF+photoz':
        plt.clf()
        title = ['Random Forest', '', 'Photo-z']
        plot(z, rf, 'PDF', 'Random Forest', outdir)
        plot(z, photo_z, 'PDF', title_photoz, outdir, outname='photoz')

        product = np.multiply(rf.T, photo_z)
        product = product.T
        # norm_product = product/product.sum()

        x = [z, [], z, z]
        y = [rf, [], photo_z, product]
        subplot(x, y, title, outdir, title_photoz)

    elif final_pdf == 'SF+photoz':
        plt.clf()
        title = ['', 'Survival Function', 'Photo-z']
        plot(z, sf, '1 - CDF', 'Survival Function', outdir)
        plot(z, photo_z, 'PDF', title_photoz, outdir, outname='photoz')

        product = np.multiply(sf.T, photo_z)
        product = product.T
        # norm_product = product/product.sum()

        x = [[], z, z, z]
        y = [[], sf, photo_z, product]
        subplot(x, y, title, outdir, title_photoz)

    elif final_pdf == 'RF':
        plt.clf()
        plot(z, rf, 'PDF', 'Random Forest', outdir)

    elif final_pdf == 'SF':
        plt.clf()
        plot(z, sf, '1 - CDF', 'Survival Function', outdir)

    elif final_pdf == 'photoz':
        plt.clf()
        plot(z, photo_z, 'PDF', title_photoz, outdir, outname='photoz')



import gzip
import cPickle
import numpy as np


def load(file_name):
    """Loads a compressed object from disk"""
    file = gzip.GzipFile(file_name, 'rb')
    object = cPickle.load(file)
    file.close()
    return object


def save(object, file_name, protocol=-1):
    """Saves a compressed object to disk"""
    file = gzip.GzipFile(file_name, 'wb')
    cPickle.dump(object, file, protocol)
    file.close()
    return ()


def mask(my_flux, percent):
    """Removes outliers. Only keeps data within input percentile"""
    outlier_mask = my_flux < np.percentile(my_flux, percent)
    flux = my_flux[outlier_mask]
    return flux


def mask3d(x, y, z, percent):
    outlier_indices = [[], [], []]
    for i, flux in enumerate([x, y, z]):
        outlier_mask = flux < np.percentile(flux, percent)
        outlier_indices[i] = np.where(outlier_mask==False)[0]
    outlier_indices = list(set(np.ravel(outlier_indices)))

    x_masked = np.delete(x, outlier_indices)
    y_masked = np.delete(y, outlier_indices)
    z_masked = np.delete(z, outlier_indices)

    return x_masked, y_masked, z_masked


def add_error(mc_dict, filter, flux_filter_err):
    for sne_type in mc_dict.keys():
        mc_dict[sne_type]['flux'][filter] += np.random.normal(
         loc=0, scale=flux_filter_err,
         size=np.shape(mc_dict[sne_type]['flux'][filter]))
    return mc_dict


def dict_from_list(list_of_dicts):
    ds = list_of_dicts
    d = {}
    for k in ds[0].iterkeys():
        d[k] = tuple(d[k] for d in ds)
    return d


def file_name(my_dir, file_dir, z):
    z = z * 100
    file_z = '%.0f' % z
    if len(file_z) < 3:
        file_z = '0' + file_z
    name = my_dir + file_dir + 'z' + file_z + '_' + 'simulated_mc.gz'
    return name



import os
import time
import sncosmo
import argparse
import numpy as np
from scipy import io, interpolate, optimize
from numpy.random import seed, normal, uniform, choice
from funcsForSimulatedData import save, dict_from_list

outdir = '/Users/carolinesofiatti/projects/SingleEpochPhotoClass/temp/'
dust = sncosmo.CCM89Dust()
hostr_v = 3.1
seed(11)
t0 = 0

parser = argparse.ArgumentParser()
parser.add_argument(
                dest='n', type=int,
                help='Number of MC itirations ex.:1000')
parser.add_argument(
                dest='phase_min', type=int,
                help='Minimum phase ex.: 0')
parser.add_argument(
                dest='phase_max', type=int,
                help='Maximum phase ex.: 5')
parser.add_argument(
                dest='z_min', type=float,
                help='Minimum redshift ex.: 0.5')
parser.add_argument(
                dest='z_max', type=float,
                help='Maximum redshift ex.: 2.05')
parser.add_argument(
                dest='z_interval', type=float,
                help='Redshift interval ex.: 0.05')
parser.add_argument(
                dest='filters', type=list, nargs='?',
                default=['f105w', 'f140w', 'f160w', 'uvf814w'],
                help='List of strings where the strings are filter names.')
args = parser.parse_args()

filters = parser.get_default('filters')

zero_point = {'f105w': 26.235, 'f140w': 26.437, 'f160w': 25.921,
              'uvf814w': 25.0985, 'zpsys': 'ab'}

all_model_Ibc = ['s11-2005hl', 's11-2005hm', 's11-2006fo', 'nugent-sn1bc',
                 'nugent-hyper', 's11-2006jo', 'snana-2004fe',
                 'snana-2004gq', 'snana-sdss004012', 'snana-2006fo',
                 'snana-sdss014475', 'snana-2006lc', 'snana-04d1la',
                 'snana-04d4jv', 'snana-2004gv', 'snana-2006ep',
                 'snana-2007y', 'snana-2004ib', 'snana-2005hm',
                 'snana-2006jo', 'snana-2007nc']


all_model_II = ['s11-2005lc', 's11-2005gi', 's11-2006jl', 'nugent-sn2p',
                'snana-2004hx', 'snana-2005gi', 'snana-2006gq',
                'snana-2006kn', 'snana-2006jl', 'snana-2006iw',
                'snana-2006kv', 'snana-2006ns', 'snana-2007iz',
                'snana-2007nr', 'snana-2007nr', 'snana-2007kw',
                'snana-2007ky', 'snana-2007lj', 'snana-2007lb',
                'snana-2007ll', 'snana-2007nw', 'snana-2007ld',
                'snana-2007md', 'snana-2007lz', 'snana-2007lx',
                'snana-2007og', 'snana-2007ny', 'snana-2007nv',
                'snana-2007pg', 's11-2004hx', 'nugent-sn2l', 'nugent-sn2n',
                'snana-2006ez', 'snana-2006ix']


def all_mabs(sne_type):
    # The majority of Type II are IIp
    my_mabs = {'Ibc': normal(-17.6, scale=1),
               'II': normal(-16.80, scale=0.97),
               'IIn': normal(-18.62, scale=1.48),
               'IIl': normal(-17.98, scale=0.9)}
    return(my_mabs[sne_type])


def which_salt(z):
    salt_name = 'salt2'
    salt_version = '2.4'

    rest_salt_max_wav = 9200
    rest_salt_min_wav = 2000

    salt_max_wav = (1 + z) * rest_salt_max_wav
    salt_min_wav = (1 + z) * rest_salt_min_wav

    for filter in filters:
        band = sncosmo.get_bandpass(filter)
        if (band.wave[0] < salt_min_wav or band.wave[-1] > salt_max_wav):
            salt_name = 'salt2-extended'
            salt_version = '1.0'
            break
    return salt_name, salt_version


def get_model_Ia(z, min_phase, max_phase):
    """Given a filter, redshift z at given phase, generates the observed
    magnitude for SNe Type Ia"""

    alpha = 0.12
    beta = 3.
    x1 = normal(0., 1.)
    c = normal(0., 0.1)
    my_phase = uniform(min_phase, max_phase)
    mabs = normal(-19.1 - alpha*x1 + beta*c, scale=0.15)
    salt_name, salt_version = which_salt(z)
    model_Ia = sncosmo.Model(source=sncosmo.get_source(salt_name,
                                                       version=salt_version))
    model_Ia.set(z=z)
    model_Ia.set_source_peakabsmag(mabs, 'bessellb', 'vega')
    p = {'z': z, 't0': t0, 'x1': x1, 'c': c}
    p['x0'] = model_Ia.get('x0')
    model_Ia.set(**p)
    p['salt_name'] = salt_name
    p['salt_version'] = salt_version
    return model_Ia, p, my_phase


def get_model_cc(z, sn_type, all_model):
    """Given a filter and redshift z, generates the observed
    magnitude for SNe Type Ibc or Type II"""
    my_model = choice(all_model)
    if my_model in ['s11-2004hx', 'nugent-sn2l']:
        sn_type = 'IIl'
    elif my_model in ['nugent-sn2n', 'snana-2006ez', 'snana-20069ix']:
        sn_type = 'IIn'
    model = sncosmo.Model(source=sncosmo.get_source(my_model),
                          effects=[dust], effect_names=['host'],
                          effect_frames=['rest'])
    mabs = all_mabs(sn_type)
    model.set(z=z)
    model.set_source_peakabsmag(mabs, 'bessellb', 'vega')
    p = {'z': z, 't0': t0, 'hostebv': uniform(-0.1, 0.65), 'hostr_v': hostr_v}
    model.set(**p)
    p['model_name'] = my_model
    my_phase = uniform(0., 5.)
    return model, p, my_phase


def get_flux(z, filter, sn_type, model, my_phase):
    zpsys = zero_point['zpsys']
    phase = my_phase + model.source.peakphase('bessellb')
    zp = zero_point[filter]
    obsflux = model.bandflux(filter, t0+(phase * (1+z)), zp, zpsys)
    return obsflux


def z_from_photo_z(photo_z_file, n, my_z_array=None):
    my_p_z = io.readsav(photo_z_file)
    pz = my_p_z['p_z']
    if my_z_array is None:
        z = np.arange(0, 5, .01)
    else:
        z = my_z_array
    if np.shape(pz) != np.shape(z):
        raise ValueError("p_z array and z array are different sizes")
    dz = z[1] - z[0]
    pz /= (dz * pz).sum()
    ecdf = np.cumsum(pz * dz)
    cdf = interpolate.interp1d(z, ecdf)

    def func(x, *args):
        my_cdf = args[0]
        cdf = args[1]
        return abs(my_cdf - cdf(x))
    out_z = []
    for i in range(n):
        my_cdf = np.random.uniform(0, 1)
        my_z = optimize.fmin(func, (1.5), args=(my_cdf, cdf), disp=0)
        out_z.append(my_z[0])
    out_z = np.asarray(out_z)
    return out_z


def mc_file(n, z=None, photo_z_file=None,
            z_for_photo_z_file=None):
    """Returns 3 arrays. For each SNe type, returns an array with the observed
    filter flux.

    z is either an array (for photo_z_file) or a number"""
    files_dir = outdir + 'n%.0f_phase_%.0f_%.0f' % (n, args.phase_min,
                                                    args.phase_max)
    if not os.path.exists(files_dir):
        os.makedirs(files_dir)

    if z is None:
        z_array = z_from_photo_z(photo_z_file, n, z_for_photo_z_file)
        new_fname = files_dir + '/simulated_mc.gz'

    else:
        z_array = np.ones(n)*z
        fname = files_dir + '/z%0.2f_simulated_mc.gz' % z_array[0]
        # now we want to remove the dot
        i = fname.index('.')
        new_fname = fname[:i] + fname[i+1:]

    all_fluxes = np.empty([len(filters), 3*n])
    mc_dict = {'type_Ia': {'flux': {}, 'params': {}},
               'type_Ibc': {'flux': {}, 'params': {}},
               'type_II': {'flux': {}, 'params': {}}}
    sorted_keys = sorted(mc_dict, key=str.lower)

    p_Ia, p_Ibc, p_II = ([], [], [])

    for i in range(n):
        if i % (n/100) == 0:
            print i/(n/100), '% complete'

        my_model_Ia, my_p_Ia, my_phase_Ia = get_model_Ia(
            z_array[i], args.phase_min, args.phase_max)
        p_Ia.append(my_p_Ia)
        my_model_Ibc, my_p_Ibc, my_phase_Ibc = get_model_cc(
            z_array[i], 'Ibc', all_model_Ibc)
        p_Ibc.append(my_p_Ibc)
        my_model_II, my_p_II, my_phase_II = get_model_cc(
            z_array[i], 'II', all_model_II)
        p_II.append(my_p_II)

        for j, filter in enumerate(filters):
            all_fluxes[j, :][i] = get_flux(
                z, filter, 'Ia', my_model_Ia, my_phase_Ia)
            all_fluxes[j, :][i+n] = get_flux(
                z, filter, 'Ibc', my_model_Ibc, my_phase_Ibc)
            all_fluxes[j, :][i+(2*n)] = get_flux(
                z, filter, 'II', my_model_II, my_phase_II)

    for i, key in enumerate(sorted_keys):
        mc_dict[key]['flux'] = dict(zip(filters, [
            all_fluxes[0, i*n:(i*n+n)], all_fluxes[1, i*n:i*n+n],
            all_fluxes[2, i*n:i*n+n], all_fluxes[3, i*n:i*n+n]]))
    mc_dict['type_Ia']['params'] = dict_from_list(p_Ia)
    mc_dict['type_Ibc']['params'] = dict_from_list(p_Ibc)
    mc_dict['type_II']['params'] = dict_from_list(p_II)
    save(mc_dict, new_fname)

all_z = np.arange(args.z_min, args.z_max, args.z_interval)
for i in all_z:
    start = time.time()
    mc_file(args.n, i)
    print 'It took', time.time() - start, 'seconds.'



import glob
import gzip
import cPickle as pickle
import numpy as np
from photoz import find_photo_z
from randomForest import obtain_proba
from survivalFunc import find_percentile


def iterator(type, my_dir, file_dir, filter1, filter2, filter3,
             flux_filter1, flux_filter2, flux_filter3, flux_filter1_err,
             flux_filter2_err, flux_filter3_err, photo_z_type=None,
             photo_z_file=None, photo_z_redshift_file=None, mu=None,
             sigma=None):
    ''' type: RF (random forest)
              survival (survival function)
              photo_z
    '''
    files = []
    my_z = []
    quant = []
    my_files = glob.glob(my_dir + file_dir + '*simulated*.gz')
    files.append(sorted(my_files))

    for a in files[0]:
        min_index = a.index('/z') + 2
        my_z.append(a[min_index:min_index + 3])
    z = np.asarray(my_z, dtype=int)
    z = np.divide(z, 100.)

    if type == 'RF':
        for i in range(len(files[0])):
            RF = obtain_proba(
              my_dir, file_dir, filter1, filter2, filter3, flux_filter1,
              flux_filter2, flux_filter3, flux_filter1_err, flux_filter2_err,
              flux_filter3_err, z[i])[0]
            quant.append(RF)

    elif type == 'survival':
        for i in range(len(files[0])):
            survival_f = find_percentile(my_dir, file_dir, filter1, filter2,
                                         filter3, flux_filter1, flux_filter2,
                                         flux_filter3, flux_filter1_err,
                                         flux_filter2_err, flux_filter3_err,
                                         z[i])
            quant.append(survival_f)

    elif type == 'photo_z':
        if photo_z_type == 'file':
            photo_z_file = photo_z_file
            photo_z = find_photo_z(photo_z_type, photo_z_file,
                                   photo_z_redshift_file, z, mu, sigma)
        elif photo_z_type == 'gauss':
            photo_z = find_photo_z(photo_z_type, photo_z_file,
                                   photo_z_redshift_file, z, mu, sigma)
        quant = np.asarray(photo_z)
    return (quant, z)


def save_file(object, filename, protocol=-1):
    """Saves a compressed object to disk"""
    file = gzip.GzipFile(filename, 'wb')
    pickle.dump(object, file, protocol)
    file.close()
    return ()


def save_arrays(my_dir, file_dir, filter1, filter2, filter3, flux_filter1,
                flux_filter2, flux_filter3, flux_filter1_err,
                flux_filter2_err, flux_filter3_err, outdir):

    rf, rf_z = iterator('RF', my_dir, file_dir, filter1, filter2, filter3,
                        flux_filter1, flux_filter2, flux_filter3,
                        flux_filter1_err, flux_filter2_err, flux_filter3_err)

    sf, sf_z = iterator('survival', my_dir, file_dir, filter1, filter2,
                        filter3, flux_filter1, flux_filter2, flux_filter3,
                        flux_filter1_err, flux_filter2_err, flux_filter3_err)

    keys = ['rf', 'rf_z',
            'sf', 'sf_z',
            ]
    values = [rf, rf_z, sf, sf_z]

    my_dict = dict(zip(keys, values))
    file_name = outdir + 'random-forest_survival-function.p'
    save_file(my_dict, file_name)
    return file_name



import numpy as np
from sklearn.neighbors import KernelDensity
from funcsForSimulatedData import load, file_name, add_error


def kde3d(x, y, z, data_point):
    values = np.vstack([x, y, z]).T
    # Use grid search cross-validation to optimize the bandwidth
    # params = {'bandwidth': np.logspace(-1, 1, 20)}
    kde = KernelDensity(bandwidth=0.3)
    kde.fit(values)
    kde_coords = kde.sample(10000)
    log_pdf = kde.score_samples(kde_coords)
    percentile = np.sum(log_pdf < kde.score(data_point))/10000.
    return (percentile)


def find_percentile(my_dir, file_dir, filter1, filter2, filter3, flux_filter1,
                    flux_filter2, flux_filter3, flux_filter1_err,
                    flux_filter2_err, flux_filter3_err, z):

    all_percentile = []
    data_point = (flux_filter1, flux_filter2, flux_filter3)
    data_point = np.asarray(data_point)

    mc_file_name = file_name(my_dir, file_dir, z)
    mc_dict = load(mc_file_name)
    mc_dict = add_error(mc_dict, filter1, flux_filter1_err)
    mc_dict = add_error(mc_dict, filter2, flux_filter2_err)
    mc_dict = add_error(mc_dict, filter3, flux_filter3_err)

    print 'redshift: ', z

    Ia_percentile = kde3d(mc_dict['type_Ia']['flux'][filter1],
                          mc_dict['type_Ia']['flux'][filter2],
                          mc_dict['type_Ia']['flux'][filter3], data_point)

    Ibc_percentile = kde3d(mc_dict['type_Ibc']['flux'][filter1],
                           mc_dict['type_Ibc']['flux'][filter2],
                           mc_dict['type_Ibc']['flux'][filter3], data_point)

    II_percentile = kde3d(mc_dict['type_II']['flux'][filter1],
                          mc_dict['type_II']['flux'][filter2],
                          mc_dict['type_II']['flux'][filter3], data_point)

    my_percentile = (Ia_percentile, Ibc_percentile, II_percentile)
    my_percentile = np.asarray(my_percentile)
    all_percentile.append(my_percentile)
    all_percentile = np.squeeze(all_percentile)

    return all_percentile



import copy
import pandas as pd
import numpy as np
from sklearn.ensemble import RandomForestClassifier
from funcsForSimulatedData import load, file_name, add_error


def get_data(my_dir, file_dir, filter1, filter2, filter3, flux_filter1_err,
             flux_filter2_err, flux_filter3_err, z):

    indices = [i for i, x in enumerate(file_dir) if x == '_']
    n = int(file_dir[1:indices[0]])
    n_Ibc_start = int(n)
    n_II_start = 2 * n_Ibc_start

    mc_file_name = file_name(my_dir, file_dir, z)
    mc_dict = load(mc_file_name)
    mc_dict = add_error(mc_dict, filter1, flux_filter1_err)
    mc_dict = add_error(mc_dict, filter2, flux_filter2_err)
    mc_dict = add_error(mc_dict, filter3, flux_filter3_err)

    all_filter1 = np.concatenate(
                (mc_dict['type_Ia']['flux'][filter1],
                 mc_dict['type_Ibc']['flux'][filter1],
                 mc_dict['type_II']['flux'][filter1]))

    all_filter2 = np.concatenate(
                (mc_dict['type_Ia']['flux'][filter2],
                 mc_dict['type_Ibc']['flux'][filter2],
                 mc_dict['type_II']['flux'][filter2]))

    all_filter3 = np.concatenate(
                (mc_dict['type_Ia']['flux'][filter3],
                 mc_dict['type_Ibc']['flux'][filter3],
                 mc_dict['type_II']['flux'][filter3]))

    data = [all_filter1, all_filter2, all_filter3]
    data = zip(*data)
    all_sources = pd.DataFrame(data, columns=[filter1, filter2, filter3],
                               index=range(3*n))
    all_sources['Type'] = 'Type Ia'
    all_sources.ix[n_Ibc_start:n_II_start, 'Type'] = 'Type Ibc'
    all_sources.ix[n_II_start:, 'Type'] = 'Type II'
    all_features = copy.copy(all_sources)
    all_label = all_sources["Type"]
    del all_features["Type"]

    X = copy.copy(all_features.values)
    Y = copy.copy(all_label.values)

    return X, Y


def obtain_proba(my_dir, file_dir, filter1, filter2, filter3, flux_filter1,
                 flux_filter2, flux_filter3, flux_filter1_err,
                 flux_filter2_err, flux_filter3_err, z):

    X, Y = get_data(my_dir, file_dir, filter1, filter2, filter3,
                    flux_filter1_err, flux_filter2_err, flux_filter3_err, z)

    Y[Y == "Type Ia"] = 0
    Y[Y == "Type Ibc"] = 1
    Y[Y == "Type II"] = 2
    Y = Y.astype(int)

    clf = RandomForestClassifier(n_estimators=200, oob_score=True)
    rfc = clf.fit(X, Y)
    proba = rfc.predict_proba([flux_filter1, flux_filter2, flux_filter3])
    return(proba)



from setuptools import setup, find_packages
import os

setup(name='singlEpoClass',
      version='1.2',
      description='Single Epoch Transient Classifier',
      url='http://github.com/sofiatti/SingleEpochPhotometricClassifier',
      author='Caroline Sofiatti',
      author_email='sofiatti@berkeley.edu',
      packages=find_packages(),
      scripts=['scripts/' + f for f in os.listdir('scripts')],
      zip_safe=False)
